#!/usr/bin/env python3

# This script is part #2 to reproduce the results of Bletery and Nocquet (Science, 2023)
# It extracts time series sampled at 5 min that are complete for 2 days before the earthquakes downloaded from the UNR database
# It writes the time series in ASCII files and matrices

from glob import glob
import numpy as np
import pyacs.lib.astrotime as at
from datetime import datetime
import os
import shutil

safe_time = 0# 2.5*60 
nsample = 576

leq_dir = 'station_locations'
data_dir = 'data'
data_byEQ_dir = 'data_byEQ' 
leq_file = glob(leq_dir+"/20*_MW_*.dat")

if os.path.exists(data_byEQ_dir):
    shutil.rmtree(data_byEQ_dir)
os.makedirs(data_byEQ_dir)

for eq_file in leq_file:

    print("Processing EQ %s" % eq_file.split('/')[-1][0:-4])
    eq_dir = data_byEQ_dir+'/'+eq_file.split('/')[-1][0:-4] 

    # EQ date as datetime and pyacs seconds
    date_eq_as_datetime = datetime.strptime(eq_file.split('/')[-1].split('.')[0].split('_')[0], '%Y%m%dT%H:%M:%S')
    date_eq_seconds = at.datetime2seconds(date_eq_as_datetime)

    # list of GPS sites
    if not os.stat(eq_file).st_size==0:
        np_code = np.atleast_1d(np.genfromtxt(eq_file,dtype=str,usecols=0))
        st_lon = np.atleast_1d(np.genfromtxt(eq_file,dtype=float,usecols=1))
        st_lat = np.atleast_1d(np.genfromtxt(eq_file,dtype=float,usecols=2))
    else:
        np_code = []

    # list of days to be considered
    mjd_eq = at.datetime2mjd(date_eq_as_datetime)
    (np_doy,np_year,np_ut) = at.mjd2dayno([mjd_eq-2,mjd_eq-1,mjd_eq])

    # Convert time of earthquakes in J2000 time (for consistency with GPS) 
    sec_J2000 = int(at.decyear2seconds(2000.0,rounding='second')+ 12*3600) # it looks like the definition of J2000 has been rounded to 01/01/2000 at 12:00
    date_eq_sec_J2000 = date_eq_seconds - sec_J2000
    
    Nst = 0 # initiate the count of complete time series
    
    # loop on code
    lcode_good, llon_good, llat_good = [], [], []
    for code in np_code:
        try:
            ts0 = np.genfromtxt((data_dir+"/%s.%04d.%03d.kenv.gz" % (code,np_year[0],np_doy[0])), skip_header=1)
            ts1 = np.genfromtxt((data_dir+"/%s.%04d.%03d.kenv.gz" % (code,np_year[1],np_doy[1])), skip_header=1)
            ts2 = np.genfromtxt((data_dir+"/%s.%04d.%03d.kenv.gz" % (code,np_year[2],np_doy[2])), skip_header=1)
        except:
            continue

        ts = np.concatenate((ts0,ts1,ts2),axis=0)[:,1:11]
        np_sec_J2000 = ts[:,0].astype(int)
        np_sec_wrt_eq = np_sec_J2000 - (date_eq_sec_J2000-safe_time) 

        idx = np.where((np_sec_wrt_eq < 0))[0][-1]
        if np_sec_wrt_eq[idx] < -300: 
            continue

        # select 48h of complete 5min samples (576 samples) before the EQ time origin
        try:
            sample_date = np_sec_J2000[idx-nsample:idx]
            maxsamp = np.max(np.diff(sample_date))
        except:
            continue

        if maxsamp > 300: # check that the time between 2 samples is not > 300 s (i.e. that there is no missing sample)
            continue
        if len(sample_date) < nsample: # check if the time series has 576 samples
            continue

        Nst = Nst + 1    
        ts = ts[idx-nsample:idx,:]
        
        if not os.path.exists(eq_dir):
            os.makedirs(eq_dir) # creates directories for each earthquake
        out_name = (eq_dir+'/TS_'+code+'.dat') 
        fmt = "%10d %6d %6d %4d %4d %03d %05d %10.6lf %10.6lf %10.6lf"
        np.savetxt(out_name,ts,fmt=fmt)
        
        I = np.where(np_code==code)
        lcode_good.append(code)
        llon_good.append(st_lon[I])
        llat_good.append(st_lat[I])
 
    print("%d complete time series found\n" % Nst)

    if Nst > 0:
        with open(eq_dir+'/station_list.txt', 'w') as fp:
            for j in range(len(lcode_good)):
                fp.write("%s " % lcode_good[j])
                fp.write("%f " % llon_good[j])
                fp.write("%f\n" % llat_good[j])
